TCPServer = TCPServer or {}

--@TCPServer:new creates a tcp server for communicating with the clients/Drivers
--@param objTcpServer.communicator is the communicator class
--created 11.08.2020 09:45 Sebastian Steininger, changed 11.08.2020 13:12 Sebastian Steininger
function TCPServer:new(communicator_)
    local objTcpServer = {}
    setmetatable(objTcpServer, self)
    self.__index = self
    objTcpServer.communicator = communicator_
    objTcpServer.server = {
	   clients = {},
	   clientCntMap = {},
	   clientsCnt = 0,
	   
	   --stop stops the tcp server and tells his clients to disconnect before doing that
	   --created 11.08.2020 09:45 Sebastian Steininger, changed 12.08.2020 09:56 Sebastian Steininger
	   stop = function(self)
		  if (self.socket ~= nil) then
			 self.socket:Close()
			 self.socket = nil
			 for i=1,self.clientsCnt,1 do
				cli = self.clients[i]
				Dbg:Info("Disconnecting " .. cli:GetLocalAddress().ip .. ":" .. cli:GetLocalAddress().port)
				cli:Close(true)
			 end
			 self.clients = {}
			 self.clientCntMap = {}
			 self.clientsCnt = 0
		  end
	   end,
	   
	   --start starts the tcp server
	   --created 11.08.2020 09:45 Sebastian Steininger, changed 11.08.2020 13:12 Sebastian Steininger
	   start = function(self, maxClients, bindAddr, port, done)
		  local calledDone = false
		  self.socket = C4:CreateTCPServer()
		  --Events the following events are selfe explaning with their names
		  --created 11.08.2020 09:45 Sebastian Steininger, changed 11.08.2020 13:12 Sebastian Steininger
		  :OnResolve(
			 function(srv, endpoints)
				Dbg:Info("Server " .. tostring(srv) .. " resolved listening address")
				for i = 1, #endpoints do
				    Dbg:Info("Available endpoints: [" .. i .. "] ip=" .. endpoints[i].ip .. ":" .. endpoints[i].port)
				end
			 end)

		  :OnListen(
			 function(srv, endpoint)
				-- Handling this callback is optional.  It merely lets you know that the server is now actually listening.
				local addr = srv:GetLocalAddress()
				Dbg:Trace("Server " .. tostring(srv) .. " choose endpoint " .. endpoint.ip .. ":" .. endpoint.port .. ", listening on " .. addr.ip .. ":" .. addr.port)
				    if (not calledDone) then
					   calledDone = true
					   done(true, addr)
				    end
			 end)

		  :OnError(
			 function(srv, code, msg, op)
				-- code is the system error code (as a number)
				-- msg is the error message as a string
				Dbg:Info("Server " .. tostring(srv) .. " Error " .. code .. " (" .. msg .. ")")
				if (not calledDone) then
				    calledDone = true
				    done(false, msg)
				end
			 end)

		  :OnAccept(
			 function(srv, client)
				-- srv is the instance C4:CreateTCPServer() returned
				-- client is a C4LuaTcpClient instance of the new connection that was just accepted
				Dbg:Info("Connection on server " .. tostring(srv) .. " accepted, client: " .. tostring(client))
				if (self.clientsCnt >= maxClients) then
				    client:Write("Sorry, I only allow " .. maxClients .. " concurrent connections!\r\n"):Close(true)
				    return
				end
				self.clientsCnt = self.clientsCnt + 1
				self.clients[self.clientsCnt] = client
				self.clientCntMap[client] = self.clientsCnt
				client
				    :OnRead(
					   function(cli, data)
						  data = string.sub(data, 1, -3) -- Cut off \r\n
						  Dbg:Trace(data)
						  if(string.sub(data, 1,3) == "lc/") then
							 Dbg:Debug("local command received")
							 if(string.match(data, "lc/register/"))then
								local uuid = string.sub(data, 13)
								Dbg:Trace("uuid: "..uuid)
								objTcpServer.communicator:RegisterReceiveClient(uuid,client)
							 elseif(string.match(data, "lc/enableStatusUpdate"))then
								objTcpServer.communicator:EnabledStatusUpdateClient(client)
							 end
						  else
							 Dbg:Debug("TCP-Server got command, send to mini-server")
							 objTcpServer.communicator:SendCommand(data)
						  end
						  cli:ReadUntil("\r\n")
					   end
				    )
				    
				    :OnWrite(
					   function(cli)
						  -- cli is the C4LuaTcpClient instance (same as client in the OnAccept handler).  This callback is called when
						  -- all data was sent.
						  Dbg:Trace("Server " .. tostring(srv) .. " Client " .. tostring(client) .. " Data was sent.")
					   end
				    )

				    :OnDisconnect(
					   function(cli, errCode, errMsg)
						  -- cli is the C4LuaTcpClient instance (same as client in the OnAccept handler) that the data was read on
						  -- errCode is the system error code (as a number).  On a graceful disconnect, this value is 0.
						  -- errMsg is the error message as a string.
						  if (errCode == 0) then
							 Dbg:Trace("Server " .. tostring(srv) .. " Client " .. tostring(client) .. " Disconnected gracefully.")
						  else
							 Dbg:Trace("Server " .. tostring(srv) .. " Client " .. tostring(client) .. " Disconnected with error " .. errCode .. " (" .. errMsg .. ")")
						  end
						  --if (info.name ~= nil) then
						  --	self:notifyOthers(cli, info.name .. " disconnected!\r\n")
						  --end
						  if(self.clients[self.clientCntMap[cli]]) then
							 self.clients[self.clientCntMap[cli]] = nil
							 self.clientCntMap[cli]=nil
							 self.clientsCnt = self.clientsCnt - 1
						  end
					   end
				    )
				    :OnError(
					   function(cli, code, msg, op)
						  -- cli is the C4LuaTcpClient instance (same as client in the OnAccept handler) that the data was read on
						  -- code is the system error code (as a number)
						  -- msg is the error message as a string
						  -- op indicates what type of operation failed: "read", "write"
						  Dbg:Info("Server " .. tostring(srv) .. " Client " .. tostring(client) .. " Error " .. code .. " (" .. msg .. ") on " .. op)
					   end
				    )
				    :ReadUntil("\r\n")
				end
			 )
			 :Listen(bindAddr, port)
			 if (self.socket ~= nil) then
				return self
			 end
		  end
	   }
    return objTcpServer
end

--TCPServer:start starts the tcp server
--created 11.08.2020 09:45 Sebastian Steininger, changed 26.08.2020 13:53 Sebastian Steininger
function TCPServer:Start()
    -- Start the server with a limit of 100 concurrent connections
    self.server:start(100, "!local", 40, function(success, info)
	   if (success) then
		  Dbg:Info("Server listening on " .. info.ip .. ":" .. info.port)
	   else
		  Dbg:Info("Could not start server: " .. info)
	   end
    end)
end

--TCPServer:sendToAllClients sends a message to all connected clients
--@param strDate is the data that is send to all connected clients
--created 11.08.2020 09:45 Sebastian Steininger, changed 11.08.2020 13:12 Sebastian Steininger
function TCPServer:sendToAllClients(strData)
    strData = strData.."\r\n"
    Dbg:Trace("Send to all Clients")
    for i=1,self.server.clientsCnt,1 do
	   self.server.clients[i]:Write(strData)
    end
end

--TCPServer:stop stops the tcp server
--created 11.08.2020 09:45 Sebastian Steininger, changed 11.08.2020 13:12 Sebastian Steininger
function TCPServer:Stop()
    Dbg:Debug("stop TCPServer")
    self.server:stop()
end

--TCPServer:SetCommunicatorObj sets the communicator object
--created 12.08.2020 11:22 Sebastian Steininger, changed 12.08.2020 11:22 Sebastian Steininger
function TCPServer:SetCommunicatorObj(communicator)
    self.communicator = communicator
end