---------------------------------------------- COMMUNICATING WITH THE MINISERVER ----------------------------------------------
JSON = require ('common.json')

common_lib = require ('common.common_lib')
common_timer = require ('common.common_timer')
common_handlers = require ('common.common_handlers')
common_websocket = require ('common.common_websocket')
tcpServer = require('TCPServer')

require ('urlCoder')
require ('messageHandler')
require ('loxApp3JsonHandler')

---------------------------------------- Class Communicator ----------------------------------------
Communicator = Communicator or {}

--only until connectionProtocoll is changed than aes and rsa are not necessary

local aesKey = Properties["aesKey"]
local aesIv = Properties["aesIv"]

--@Communicator:new creates a websoket for communicating with the miniserver
--@details if this function was already called once it will close the first socket, can only handle one websoket connection
--@param strDomainIp is the ip/Domain of the miniserver
--@param port is the port of the miniserver (default 443 for wss)
--created 28.07.2020 09:45 Sebastian Steininger, changed 25.08.2020 10:43 Sebastian Steininger
function Communicator:new(strDomainIp_, strPort_, strUser_, strPassword_, fnRestart)
    local objCommunicator = {}
    setmetatable(objCommunicator, self)
    self.__index = self
    objCommunicator.strUser = strUser_
    objCommunicator.strPassword = strPassword_
    objCommunicator.strDomainIp = strDomainIp_
    objCommunicator.strPort = strPort_
    local fncKeepAlive = function()
	   objCommunicator:KeepAlive()
    end
    objCommunicator.messageHandler = MessageHandler:new(fnRestart, fncKeepAlive)
    objCommunicator.tabRegisteredReceiveClient ={}
    objCommunicator.encSesKey = Properties["encryptedSessionKey"]
    objCommunicator.tcpServer = nil
    objCommunicator.tabEnabledStatusUpdateClients = {}
    return objCommunicator
end
--Communicator:AddTCPServer adds the tcp server
--created 12.08.2020 11:13 Sebastian Steininger, changed 12.08.2020 11:13 Sebastian Steininger
function Communicator:AddTCPServer(tcpServer)
    self.tcpServer = tcpServer
end

--@Communicator:SendCommand sends a command to the miniserver
--@details Tests if the command is not an empty sring
--@param strCommand is the command that sould be send
--created 28.07.2020 09:23 Sebastian Steininger, changed 07.08.2020 09:18 Sebastian Steininger
function Communicator:SendCommand(strCommand)
    strCommand = strCommand or "keepalive"
    if (self.webSocketLoxoneMiniserver) then
	   Dbg:Info("Sending a command to the server, command: "..strCommand)
	   self.webSocketLoxoneMiniserver:Send (strCommand)
    else
	   Dbg:Alert("Websocket does not exist!")
    end
end

--@Communicator:CloseConnection closes the websocket connection
--created 28.07.2020 09:50 Sebastian Steininger, changed 06.08.2020 08:19 Sebastian Steininger
function Communicator:CloseConnection()
    Dbg:Info("Closeing the websocket connection")
    if(self.webSocketLoxoneMiniserver) then
	   self.webSocketLoxoneMiniserver:delete()
    end
end

--@Communicator:RegisterReceiveClient registers a client for a uuid
--created 12.08.2020 09:50 Sebastian Steininger, changed 12.08.2020 14:18 Sebastian Steininger
function Communicator:EnabledStatusUpdateClient(client)
    Dbg:Debug("register enabledStatusUpdate client")
    table.insert(self.tabEnabledStatusUpdateClients, client)
end

--@Communicator:RegisterReceiveClient registers a client for a uuid
--created 12.08.2020 15:44 Sebastian Steininger, changed 12.08.2020 15:45 Sebastian Steininger
function Communicator:RegisterReceiveClient(uuid, client)
    Dbg:Debug("register receive client")
    if(self.tabRegisteredReceiveClient[uuid]) then
	   Dbg:Debug("register first receive client")
	   table.insert(self.tabRegisteredReceiveClient[uuid], client)
    else
	   Dbg:Debug("register multiple receive client")
	   local tabClients = {}
	   table.insert(tabClients, client)
	   self.tabRegisteredReceiveClient[uuid] = tabClients
    end
end

--Communicator:CreateWebsocket creates a new websocket connection to the loxone miniserver
--@param value is wss://ipOrUrl:Port/wss/rfc6455 
--created 28.07.2020 14:32 Sebastian Steininger, changed 03.08.2020 08:29 Sebastian Steininger
function Communicator:CreateWebsocket(strDomainIp)
    if (self.webSocketLoxoneMiniserver) then
	   self.webSocketLoxoneMiniserver:delete ()
    end
    
    self.webSocketLoxoneMiniserver = WebSocket:new (strDomainIp, {"Sec-WebSocket-Protocol: remotecontrol"})
    
    local fncReceivedData = function(selfWs, data)
	   self:ReceivedData(selfWs, data)
    end
    self.webSocketLoxoneMiniserver:SetProcessMessageFunction (fncReceivedData)
    
    local fncEstablishedConnection = function(selfWs)
	   self:EstablishedConnection(selfWs)
    end
    self.webSocketLoxoneMiniserver:SetEstablishedFunction (fncEstablishedConnection)
    
    local fncConnectionClosedByRemote= function(selfWs)
	   self:ConnectionClosedByRemote(selfWs)
    end
    self.webSocketLoxoneMiniserver:SetClosedByRemoteFunction (fncConnectionClosedByRemote)

    self.webSocketLoxoneMiniserver:Start ()
end

--@Communicator:ReceivedData handles received data
--@param data is the received data from the miniserver
--created 28.07.2020 09:55 Sebastian Steininger, changed 12.08.2020 14:29 Sebastian Steininger
function Communicator:ReceivedData(selfWs, data)
    Dbg:Info("Data received")
    
    if(self.nResponseEncryption == true) then
	   data = DecryptData(data, aesKey, aesIv)
	   Dbg:Info("ReceivedEncData")
    else
	   Dbg:Info("ReceivedData")
    end
    
    if(string.match(data, "gettoken") and string.match(data, "200")) then
	   Dbg:Info("token received")
	   self.nResponseEncryption = false
	   local arrGetToken = C4:JsonDecode(data)
	   self.strToken = arrGetToken.LL.value.token
	   Dbg:Info("EnableStatusUpdate")
	   self:EnableStatusUpdate()
	   return
    end
    
    for index, client in pairs(self.tabEnabledStatusUpdateClients) do
	   client:Write(data.."\r\n")
    end
        
    arrMessages, nNumberOfProcessedMessages = self.messageHandler:Resolve(data)
    for i=1,nNumberOfProcessedMessages,1 do
	   Dbg:Debug("Handle message for notifying a client")
	   Dbg:Debug(arrMessages[i].strUuid)
	   if(arrMessages[i].strUuid ~= "none") then
		  Dbg:Debug("arrMessage[i].strUuid ~= none")
		  if(self.tabRegisteredReceiveClient[arrMessages[i].strUuid] ~= nil) then
			 Dbg:Debug("to this uuid, clients are registered")
			 for index, client in pairs(self.tabRegisteredReceiveClient[arrMessages[i].strUuid]) do
				Dbg:Debug("Send message json encoded to a connected registered client")
				messageJson = tostring(C4:JsonEncode(arrMessages[i], true))
				Dbg:Debug(messageJson)
				client:Write(messageJson.."\r\n")
				Dbg:Debug("Message sent")
			 end
		  end
	   end
    end
end

--@Communicator:EstablishedConnection called when the connection is established
--created 28.07.2020 09:55 Sebastian Steininger, changed 07.08.2020 09:32 Sebastian Steininger
function Communicator:EstablishedConnection(selfWs)
    Dbg:Info("EstablishedConnection")
    --make the key exchange, will be not necessary after the conProt. is changed
    Dbg:Trace("Keyexchange")
    self:SendCommand("jdev/sys/keyexchange/"..self.encSesKey)
    Dbg:Debug("keyExchange command send")
    local fncGetTokenAndUserSalt = function(ticketId, strValue)
	   self:GetTokenAndUserSalt(ticketId, strValue)
    end
    C4:urlGet('https://'..self.strDomainIp..":"..self.strPort.."/jdev/sys/getkey2/"..self.strUser, {}, false, fncGetTokenAndUserSalt)
end

--@Communicator:ConnectionClosedByRemote called when the connection is closed by the miniserver
--created 28.07.2020 09:55 Sebastian Steininger, changed 03.08.2020 08:29 Sebastian Steininger
function Communicator:ConnectionClosedByRemote(self)
    Dbg:Info("ConnectionClosedByRemote")
end

--@Communicator:ConnectToMiniserver starts the connection process
--created 28.07.2020 11:07 Sebastian Steininger, changed 03.08.2020 09:45 Sebastian Steininger
function Communicator:ConnectToMiniserver()
    local fncGetApiKey= function (ticketId, data)
	   self:GetApiKey(ticketId, data)
    end
    Dbg:Debug("Get api-key")
    C4:urlGet('https://'..self.strDomainIp..":"..Properties["Port"].."/jdev/cfg/apiKey", {}, false, fncGetApiKey)
end

--@Communicator:GetApiKey This function is cald async from urlGet in "ConnectToMiniserver" and deals with the respnse
--@param ticketId is the id of the async received data
--@param strValue is the received data
--created 28.07.2020 11:12 Sebastian Steininger, changed 03.08.2020 09:59 Sebastian Steininger
function Communicator:GetApiKey(ticketId, strValue)
    Dbg:Debug("getApiKey: "..strValue)
    -- Decode the json in two steps because ' is not workig
    local arrApiKey = C4:JsonDecode(strValue)
    -- replace ' with " because ' is not working and replacing it with one decode does not work
    local strValue = string.gsub(arrApiKey.LL.value, '\'', '"')
    local arrApiKeyValue = C4:JsonDecode(strValue)

    self.strApiKey = arrApiKeyValue.key
    self.strVersion = arrApiKeyValue.version
    self.strSnr = arrApiKeyValue.snr
    
    Dbg:Trace ("----Values----")
    Dbg:Trace("apiKey: " .. self.strApiKey)
    Dbg:Trace("version " .. self.strVersion)
    Dbg:Trace("snr " .. self.strSnr)
    
    self:CreateWebsocket("wss://"..self.strDomainIp..":"..self.strPort.."/wss/rfc6455") 
end

--@getTokenAndUserSalt deals with the response, this will not be necessary if the connectionProcess changes (simple https authentication with wss)
--@param ticketId is the id of the async received data
--@param strValue is the received data
--created 28.07.2020 12:44 Sebastian Steininger, changed 03.08.2020 10:00 Sebastian Steininger
function Communicator:GetTokenAndUserSalt(ticketId, strValue)
    local arrUserSalt = C4:JsonDecode(strValue)
    self.strUserSalt = arrUserSalt.LL.value.salt
    local strHashKey = arrUserSalt.LL.value.key
    Dbg:Trace("hashKey: "..strHashKey)
    Dbg:Trace("userSalt: "..self.strUserSalt)
    local strPasswordHash = string.upper(C4:Hash("SHA1", self.strPassword..":"..self.strUserSalt))
    Dbg:Trace("pwHash: "..strPasswordHash)
    local strHash = HashHMACSH1(self.strUser..":"..strPasswordHash, strHashKey)
    --TryFromCont4
    local strRequestTokenCommand = "jdev/sys/gettoken/"..strHash.."/"..self.strUser.."/2/"..GetRandomUuid().."/"..Properties["DeviceName"]
    Dbg:Debug("requestToken: "..strRequestTokenCommand)
    local strEncCommand = EncryptCommand(strRequestTokenCommand, aesKey, aesIv, 2)
    self.nResponseEncryption = true
    self:SendCommand(strEncCommand)
end

--created 28.07.2020 15:12 Sebastian Steininger, changed 25.08.2020 11:11 Sebastian Steininger
function Communicator:KeepAlive()
    Dbg:Debug("keepalive")
    self:SendCommand("keepalive")
end

--@EnableStatusUpdate send a command to the server to receive Status-Updates
--created 28.07.2020 15:06 Sebastian Steininger, changed 07.08.2020 09:40 Sebastian Steininger
function Communicator:EnableStatusUpdate()
    self:SendCommand("jdev/sps/enablebinstatusupdate")
    local fncKeepAlive = function()
	   self:KeepAlive()
    end
    -- add a timer for keepalive
    C4:SetTimer(1000*5, fncKeepAlive, false)
    Dbg:Info("Websocket up and running")
end

---------------------------------------- END Class Communicator ----------------------------------------

--@DecryptCommand decrypts the command using aes, this will not be necessary if the connectionProcess changes (simple https authentication with wss)
--@param data is the command that sould be decrypted
--created 28.07.2020 14:48 Sebastian Steininger, changed 06.08.2020 08:29 Sebastian Steininger
function DecryptData(data, strAesKey, strAesIv)
    local arrAesOptions={}
    arrAesOptions["key_encoding"] = "HEX"
    arrAesOptions["iv_encoding"] = "HEX"
    arrAesOptions["return_encoding"] = "NONE"
    arrAesOptions["data_encoding"] = "BASE64"
    arrAesOptions["padding"] = false
    return C4:Decrypt("AES-256-CBC",strAesKey,strAesIv,data,arrAesOptions)
end

--@HashHMACSH1 is a function to hash a text with HashHMACSH1 
--@param strValue is the text that sould be hashed
--@param hashKey is the key that should be used to has the text
--created 28.07.2020 12:44 Sebastian Steininger, changed 03.08.2020 08:29 Sebastian Steininger
function HashHMACSH1(strValue, strKey)
    Dbg:Debug("Hash the strUser:pwHash")
    arrHashHMACSHA1Options ={}
    arrHashHMACSHA1Options["key_encoding"] = "HEX"
    arrHashHMACSHA1Options["return_encoding"] = "HEX"
    return C4:HMAC("SHA1", strKey, strValue , arrHashHMACSHA1Options)
end

--@EncryptCommand encrypts the command using aes, this will not be necessary if the connectionProcess changes (simple https authentication with wss)
--@param strCommand is the command that sould be encrypted
--created 28.07.2020 13:30 Sebastian Steininger, changed 03.08.2020 08:29 Sebastian Steininger
function EncryptCommand(strCommand, strAesKey, strAesIv, nEncryptionMode)
    strCipher = ConvertToLengthMulipleOf16("salt/".."F0F0".."/"..strCommand) --plaintext in Nodejs
--    print("cipher: "..cipher)
    -- AES Encrypt (cipher, key, iv)
    local arrAesOptions={}
    arrAesOptions["key_encoding"] = "HEX"
    arrAesOptions["iv_encoding"] = "HEX"
    arrAesOptions["return_encoding"] = "BASE64"
    arrAesOptions["data_encoding"] = "NONE"
    arrAesOptions["padding"] = false
    local strEncCipher, strErrorAes = C4:Encrypt("AES-256-CBC",strAesKey,strAesIv,strCipher,arrAesOptions)
    Dbg:Alert(strErrorAes)
    Dbg:Debug("encCipher: "..strEncCipher)
    if(nEncryptionMode == 2) then
	   return "jdev/sys/fenc/"..UrlCoder:UrlEncode(strEncCipher)
    elseif(nEncryptionMode == 1) then
	   return "jdev/sys/enc/"..UrlCoder:UrlEncode(strEncCipher)
    end
end

--@GetRandomUuid creates a random uuid
--@return the uuid as string
--created 03.08.2020 13:30 Sebastian Steininger, changed 03.08.2020 13:30 Sebastian Steininger
function GetRandomUuid()
    local random = math.random
    local template ='xxxxxxxx-xxxx-xxxx-xxxxxxxxxxxxxxxx'
    local uuid = string.gsub(template, '[xy]', function (c)
        local v = (c == 'x') and random(0, 0xf) or random(8, 0xb)
        return string.format('%x', v)
    end)
    return uuid
end

--@GetRandomUuid append " " to the devicename so it can be encrypted with CBC-AES256
--@return the devicename
--created 03.08.2020 13:30 Sebastian Steininger, changed 03.08.2020 13:30 Sebastian Steininger
function ConvertToLengthMulipleOf16(strCommand)
    local nSpaceAppend = 16-string.len(strCommand)%16
    for i=1,nSpaceAppend,1 do 
	   strCommand=strCommand.." "
    end
    return strCommand
end
-------------------------------------------- END COMMUNICATING WITH THE MINISERVER ---------------------------------------------