require('converter')
require('CustomTimer')

MessageHandler = MessageHandler or {}

--@MessageHandler:new creates a MessageHandler
--@returns an instance of its self
--created 28.07.2020 17:02 Sebastian Steininger, changed 25.08.2020 09:29 Sebastian Steininger
function MessageHandler:new(fnRestart, fnKeepAlive)
    local objMessageHandler = {}
    setmetatable(objMessageHandler, self)
    self.__index = self
    objMessageHandler.lastProcessedMessageHeader = Message:new()
    objMessageHandler.fnRestart = fnRestart
    objMessageHandler.fnKeepAlive = fnKeepAlive
    local fnRestart = function() 
		  fnRestart() 
		  Dbg:Debug("Restarting because no keepalive response was found.")
	   end
    objMessageHandler.initTimer = CustomTimer:new(60000, fnRestart)
    --objMessageHandler.initTimer:Start()
    return objMessageHandler
end

--@MessageHandler:Resolve handels a incoming message
--@returns a message object
--created 28.07.2020 17:07 Sebastian Steininger, changed 25.08.2020 09:34 Sebastian Steininger
function MessageHandler:Resolve(data)
    local nNumberOfProcessedMessages=1
    local arrMessages = {}
    for i=0,10000,1 do
	   Dbg:Trace("MessageHandler: Resolving a incoming message")
	   local message = Message:new()
	   --Resolve the message header
	   if(string.byte(data, 1) == 3 and string.len(data)==8) then
		  Dbg:Trace("Resolving message-header")
		  message, data = self:ResolveMessageHeader(data, message)
		  Dbg:Info("messageHeaderIdentifer: "..tostring(message.nIdentifer))
	   elseif(self.lastProcessedMessageHeader.nIdentifer==3) then
		  Dbg:Trace("Resolving Event-Table of Text-States")
		  message, data = self:ResolveEventTableOfTextStates(data, message)
	   elseif(self.lastProcessedMessageHeader.nIdentifer==2 ) then
		  Dbg:Trace("Resolving Event-Table of Value-States")
		  message, data= self:ResolveEventTableOfValueStates(data, message)
	   elseif(self.lastProcessedMessageHeader.nIdentifer==0) then
		  message.strType="Textmessage"
		  message.strText = data
		  Dbg:Info("message: "..data)
		  data = string.sub(data, 0,0)
	   elseif(self.lastProcessedMessageHeader.nIdentifer==1) then
		  Dbg:Trace("Binary File was not processed!")
		  return
	   elseif(self.lastProcessedMessageHeader.nIdentifer==4) then
		  Dbg:Trace("Event-Table of Daytimer-States was not processed!")
		  return
	   elseif(self.lastProcessedMessageHeader.nIdentifer==7) then
		  Dbg:Trace("Event-Table of Weather-States was not processed!")
		  return
	   else
		  Dbg:Warn("Message-Header with a unknown identifer was received!")
		  return
	   end
		  
	   if(message.nIdentifer==5) then
		  Dbg:Info("Out-Of-Service, connection lost (e.g. Firmware-Update)")
		  return
	   elseif(message.nIdentifer ==6) then
		  if(self.initTimer)then
			 self.initTimer:Stop()
			 self.initTimer = nil
		  end
		  Dbg:Debug("restart keepaliveTimer")
		  local fnRestart = function() 
				self.fnRestart() 
				Dbg:Debug("Restarting because keepalive did not return a response.")
			 end
		  if(self.keepaliveTimer) then
			 self.keepaliveTimer:Stop()
		  end
		  self.keepaliveTimer=CustomTimer:new(5000, fnRestart)
		  Dbg:Debug("Response keepalive, connection is up and running")
		  local fncKeepAlive = function()
				self.fnKeepAlive()
				if(self.keepaliveTimer) then
					self.keepaliveTimer:Start()
				end
			end
		  C4:SetTimer(10000, fncKeepAlive, false)
		  return
	   end
	   
	   Dbg:Info("uuid: "..tostring(message.strUuid))
	   arrMessages[nNumberOfProcessedMessages] = message
	   nNumberOfProcessedMessages = nNumberOfProcessedMessages +1
	   
	   if(string.len(data)<=0)then
		  return arrMessages, nNumberOfProcessedMessages
	   end
    end
end

--@ResolveMessageHeader handels a incoming header message
--@param message is the Message object
--@returns a message object
--created 29.07.2020 08:29 Sebastian Steininger, changed 29.07.2020 17:55 Sebastian Steininger
function MessageHandler:ResolveMessageHeader(data, message)
    message.strType = "messageHeader"
    message.nIdentifer = string.byte(data, 2)
    Dbg:Debug("Identifier")
    Dbg:Debug(message.nIdentifer)
    if(string.byte(data, 3) == 0) then
	   message.bEstimated =false
    else
    	   Dbg:Trace("Message Header: size only estimated")
	   message.bEstimated = true
    end
    message.nPayloadLen = ByteConverter:ByteToUint32LittleEdian(string.sub(data, 5,8))
    self.lastProcessedMessageHeader = message
    
    return message, string.sub(data, 9)
end

--@ResolveEventTableOfTextStates handels a incoming Event-Table of Text States (LightChange)
--@param message is the Message object
--@returns a message object
--created 29.07.2020 08:58 Sebastian Steininger, changed 29.07.2020 17:24 Sebastian Steininger
function MessageHandler:ResolveEventTableOfTextStates(data, message)
    message.strType = "EventTableOfTextStates"
    local strUuid = string.sub(data, 1,16)
    local strUuidIcon = string.sub(data, 17,32)
    local nTextLength = string.sub(data, 33,36)
    message.nTextLength = ByteConverter:ByteToUint32LittleEdian(nTextLength)
    message.strText = string.sub(data, 37,36+message.nTextLength)
    message.strUuid = ByteConverter:ByteToUuid(strUuid)
    Dbg:Trace("strText: "..message.strText)
    Dbg:Debug("uuid: "..message.strUuid)
    Dbg:Debug(message.nTextLength)
    Dbg:Debug("---------------------------------------------------")
    local nMessageEnd = 36+((message.nTextLength-1) + 4-(message.nTextLength-1)%4)
    return message, string.sub(data, nMessageEnd+1)
end

--@MessageHandler:ResolveEventTableOfValueStates handels a incoming Event-Table of Value States (LightChange)
--@param message is the Message object
--@returns a message object
--created 29.07.2020 14:44 Sebastian Steininger, changed 29.07.2020 14:45 Sebastian Steininger
function MessageHandler:ResolveEventTableOfValueStates(data, message)
    message.strType = "EventTableOfValueStates"
    local strUuid = string.sub(data, 1,16)
    message.strUuid = ByteConverter:ByteToUuid(strUuid)
    Dbg:Info("uuid: "..message.strUuid)
    local nValue = string.sub(data, 17,24)
    message.value = ByteConverter:ByteToDouble(nValue)
    return message, string.sub(data,25)
end

Message = Message or {}

--@Message:new creates a Message object for saveing the data
--@returns an instance of it selfe
--created 28.07.2020 17:02 Sebastian Steininger, changed 07.08.2020 11:29 Sebastian Steininger
function Message:new()
    local objMessage = {}           
    setmetatable(objMessage,self)  -- make Message handle lookup
    self.__index = self
    objMessage.strType = "none" --text/messageHeader/...
    objMessage.nIdentifer = "none" -- 1-7
    objMessage.bEstimated = "none" -- trur or false
    objMessage.nPayloadLen = "none" -- the lengh of the payload
    objMessage.UuidIcon = "none"
    objMessage.nTextLength = "none"
    objMessage.strText = "none"
    objMessage.nValue = "none"
    objMessage.strUuid = "none"
    return objMessage
end