require ('converter')

IntelligentRoomControllerV2 = IntelligentRoomControllerV2 or {}

--@IntelligentRoomControllerV2:new creates a new IntelligentRoomControllerV2 object
--@param communicator is the communicator object
--@param loxApp3JsonHandler is the jsonHandler object
--@return a instance of it selfe
--created 13.08.2020 8:35 Sebastian Steininger, changed 17.08.2020 09:51  Sebastian Steininger
function IntelligentRoomControllerV2:new(communicator)
    local objIntelligentRoomControllerV2 = {}
    setmetatable(objIntelligentRoomControllerV2, self)
    self.__index = self
    objIntelligentRoomControllerV2.communicator = communicator
    objIntelligentRoomControllerV2.uuid = Properties["uuid"]
    objIntelligentRoomControllerV2:RegisterReceiveFunctions()
    objIntelligentRoomControllerV2.scale = Properties["Scale"]
    return objIntelligentRoomControllerV2
end

--@IntelligentRoomControllerV2:RegisterReceiveFunctions registers the receive functions
--created 13.08.2020 8:35 Sebastian Steininger, changed 13.08.2020 8:35  Sebastian Steininger
function IntelligentRoomControllerV2:RegisterReceiveFunctions()
    local fncComfortTemperatureChange = function(message)
	   C4:UpdateProperty("comfortTemperature", tonumber(message.value))
	   C4:SendToProxy(5001,'HEAT_SETPOINT_CHANGED',{SCALE = intelligentRoomController.scale; SETPOINT= tostring(message.value)},"NOTIFY")
    end
    communicator:RegisterReceiveFunction(Properties["comfortTemperatureUuid"], fncComfortTemperatureChange)
    
    local fnccomfortTemperatureCoolChange = function(message)
	   C4:UpdateProperty("comfortTemperatureCool", tonumber(message.value))
	   C4:SendToProxy(5001,'COOL_SETPOINT_CHANGED',{SCALE = intelligentRoomController.scale; SETPOINT= tostring(message.value)},"NOTIFY")
    end
    communicator:RegisterReceiveFunction(Properties["comfortTemperatureCoolUuid"], fnccomfortTemperatureCoolChange)
    
    local fncTempActualChange = function(message)
	   C4:UpdateProperty("tempActual", tonumber(NumConverter:Round(tonumber(message.value), 1)))
	   C4:SendToProxy(5001,'TEMPERATURE_CHANGED',{TEMPERATURE=tostring(NumConverter:Round(tonumber(message.value), 1)); SCALE=self.scale},"NOTIFY")
	   Dbg:Debug("ActualTemperature set")
    end
    communicator:RegisterReceiveFunction(Properties["tempActualUuid"], fncTempActualChange)
end

--@IntelligentRoomController:SetComfortTemperature sets the comfortTemperature
--@param comfortTemperature is the comfort temperature
--created 13.08.2020 8:35 Sebastian Steininger, changed 16.07.2021 11:45  Sebastian Steininger
function IntelligentRoomControllerV2:SetComfortTemperature(comfortTemperature)
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/setOperatingMode/0")
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/stopOverride")
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/setComfortTemperature/"..tostring(comfortTemperature))
end

--@IntelligentRoomController:SetComfortTemperatureCool sets the comfortTemperatureCool
--@param comfortTemperature is the comfort temperature
--created 18.05.2021 08:51 Sebastian Steininger, changed 18.05.2021 08:51 Sebastian Steininger
function IntelligentRoomControllerV2:SetComfortTemperatureCool(comfortTemperatureCool)
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/setOperatingMode/0")
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/stopOverride")
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/setComfortTemperatureCool/"..tostring(comfortTemperatureCool))
end