ByteConverter = ByteConverter or {}

--@ByteConverter:new creates a byteConvert object
--@returns the byteConverterObject
--created 28.07.2020 16:32 Sebastian Steininger, changed 07.08.2020 10:03 Sebastian Steininger
function ByteConverter:new()
    local objByteConverter = {}           
    setmetatable(objByteConverter,self)  -- make Message handle lookup
    self.__index = self
    return objByteConverter
end

--@ByteConverter:ByteToUint32LittleEdian converts bytes to uint
--@param strByte is the string containing the bytes
--@return it returns a Uint32 number
--created 28.07.2020 16:33 Sebastian Steininger, changed 07.08.2020 16:35 Sebastian Steininger
function ByteConverter:ByteToUint32LittleEdian(strByte)
    Dbg:Debug("converting strByte to dec number (uint32)")
    local nNumberUint32=0
    for i=0,3,1 do
	   nNumberUint32 = nNumberUint32 + 256^i*string.byte(strByte, i+1)
    end
    Dbg:Debug("converting finished, Number: "..tostring(nNumberUint32))
    return nNumberUint32
end

--@ByteConverter:ByteToSring converts a byteString to a ASCI encoded string
--@param strByte is the string containing the bytes
--@return it returns the asci encoded string
--created 29.07.2020 09:07 Sebastian Steininger, changed 29.07.2020 09:07 Sebastian Steininger
function ByteConverter:ByteToSring(strByte)
    Dbg:Debug("converting strByte to string")
    local strString = ""
    for i=1,string.len(strByte),1 do
	   strString = strString..string.char(string.byte(strByte, i))..""
    end
    return strString
end

--@ByteConverter:ByteToUint16LittleEdian converts a byteString to the uuid
--@param strByte is the string containing the bytes
--@return it returns the uint16
--created 30.07.2020 08:11 Sebastian Steininger, changed 30.07.2020 08:15 Sebastian Steininger
function ByteConverter:ByteToUint16LittleEdian(strByte)
    Dbg:Debug("converting strByte to dec number (uint32)")
    local nNumberUint16=0
    for i=0,1,1 do
	   nNumberUint16 = nNumberUint16 + 256^i*string.byte(strByte, i+1)
    end
    Dbg:Debug("converting finished, Number:")
    Dbg:Debug(nNumberUint16)
    return nNumberUint16
end

--@ByteConverter:ByteToUint8LittleEdian converts a byteString to the uuid
--@param strByte is the string containing the bytes
--@return it returns the uint8
--created 30.07.2020 08:11 Sebastian Steininger, changed 30.07.2020 08:15 Sebastian Steininger
function ByteConverter:ByteToUint8LittleEdian(strByte)
    Dbg:Debug("converting strByte to dec number (uint8)")
    local nNumberUint8= string.byte(strByte, 1)
    Dbg:Debug("converting finished, Number:")
    Dbg:Debug(nNumberUint8)
    return nNumberUint8
end

--@ByteConverter:ByteToUuid converts a byteString to the uuid
--@param strByte is the string containing the bytes
--@return it returns the uuid
--created 30.07.2020 08:05 Sebastian Steininger, changed 30.07.2020 08:10 Sebastian Steininger
function ByteConverter:ByteToUuid(strByte)
    Dbg:Debug("converting strByte to Uuid")
    local Data1 = self:ByteToUint32LittleEdian(string.sub(strByte, 1,4))
    local Data2 = self:ByteToUint16LittleEdian(string.sub(strByte, 5,6))
    local Data3 = self:ByteToUint16LittleEdian(string.sub(strByte, 7,8))
    local strUuid  = HexConverter:ToFullSizeHex(string.format("%x", Data1))
    strUuid  = strUuid.."-"..HexConverter:ToFullSizeHex(string.format("%x", Data2))
    strUuid  = strUuid.."-"..HexConverter:ToFullSizeHex(string.format("%x", Data3)).."-"
    for i=1,8,1 do
	   strDecNum = self:ByteToUint8LittleEdian(string.sub(strByte, 8+i,8+i))
	   strUuid = strUuid..HexConverter:ToFullSizeHex(string.format("%x", strDecNum))
    end
    return strUuid
end

--@ByteConverter:ByteToDouble converts a byteString to double
--@param strByte is the string containing the bytes
--@return it returns the double
--created 03.08.2020 15:07 Sebastian Steininger, changed 04.08.2020 13:14 Sebastian Steininger
function ByteConverter:ByteToDouble(strByte)
    local strDoubleBinary =""
    for i=1,8,1 do
	   strDoubleBinary = strDoubleBinary..self:ByteToBits(string.sub(strByte,i,i))
    end
    Dbg:Trace("strDoubleBinary: "..strDoubleBinary)
    local s = (-1)^tonumber(string.sub(strDoubleBinary, 64,64))
    local M = self:BitsToDec(string.sub(strDoubleBinary, 1,52))
    local m = 1+M/(2^52)
    local b = 2
    local E = self:BitsToDec(string.sub(strDoubleBinary, 53,63))
    local e = E-1023
    local nDouble = s*m*b^e
    Dbg:Info("nDouble: "..tostring(nDouble))
    return nDouble
end

--@ByteConverter:ByteToBits converts a byteString to a bit string
--@param strByte is the string containing the bytes
--@return it returns the bits as string
--created 04.08.2020 13:14 Sebastian Steininger, changed 04.08.2020 13:14 Sebastian Steininger
function ByteConverter:ByteToBits(strByte)
    local nNum = string.byte(strByte)
    local strBinaryString = ""
    for i=1,8,1 do
	   if(nNum>=1)then
		  local rem = nNum%2
		  nNum = (nNum-rem)/2
		  strBinaryString = strBinaryString..tostring(rem)
	   else
		  strBinaryString = strBinaryString.."0"
	   end
    end
    return strBinaryString
end

--@ByteConverter:BitsToDec converts a bitString to a dec number
--@param strBits is the string containing the bits
--@return it returns the decimal number
--created 04.08.2020 13:57 Sebastian Steininger, changed 04.08.2020 13:57 Sebastian Steininger
function ByteConverter:BitsToDec(strBits)
    local nNum = 0
    for i=1,string.len(strBits),1 do
	   if(string.sub(strBits, i,i)=="1")then
		  nNum = nNum+2^(i-1)
	   end
    end
    return nNum
end

HexConverter = HexConverter or {}

--@HexConverter:new creates a hexConvert object
--@returns the hexConverterObject
--created 30.07.2020 09:05 Sebastian Steininger, changed 07.08.2020 09:05 Sebastian Steininger
function HexConverter:new()
    local objHexConverter = {}           
    setmetatable(objHexConverter,self)  -- make Message handle lookup
    self.__index = self
    return hexConverter
end

--@HexConverter:ToFullSizeHex converts a Hexnumber to its full size
--@param strHex the Hex number e.g. (0)e3f
--@returns the full size hex number
--created 30.07.2020 09:05 Sebastian Steininger, changed 30.07.2020 09:08 Sebastian Steininger
function HexConverter:ToFullSizeHex(strHex)
    Dbg:Debug("convert to FullSizeHex")
    local strFullSizeHex = strHex
    if(string.len(strFullSizeHex)%2==1)then
	   strFullSizeHex = "0"..strFullSizeHex
    end
    return strFullSizeHex
end

StringConverter =StringConverter or {}

--@StringConverter:new creates a stringConvert object
--@returns the stringConvert
--created 30.07.2020 13:48 Sebastian Steininger, changed 30.07.2020 13:48 Sebastian Steininger
function StringConverter:new()
    local objStringConverter = {}           
    setmetatable(objStringConverter,self)  -- make Message handle lookup
    self.__index = self
    return objStringConverter
end

--@StringConverter:StringToArray converts a String to a Table
--@detail following fotmat is requiered [1,2,3,x..]
--@param strTable ist the string that should be converted in a array
--@returns the array
--created 30.07.2020 13:48 Sebastian Steininger, changed 31.07.2020 08:00 Sebastian Steininger
function StringConverter:StringToNumArray(strArray)
    local arrMoodId = {}
    strArray = string.sub(strArray, 2,-2)
    Dbg:Trace(strArray)
    local nNumElements = 0
    Dbg:Info("try to get numArray")
    if(not string.match(strArray, ",")) then
	   arrMoodId[1] = strArray
	   return arrMoodId, 1
    end
    
    for moodID in string.gmatch(strArray, "([^"..",".."]+)") do
	   nNumElements = nNumElements+1
	   arrMoodId[nNumElements] = moodID
    end
	   
    return arrMoodId,nNumElements
end

--@StringConverter:StringToMoodArray converts a String to a Table of moods
--@detail following fotmat is requiered [{....},{....},{...]
--@param strTable ist the string that should be converted in a array
--@returns the array of JsonStrings and the length of it
--created 30.07.2020 15:00 Sebastian Steininger, changed 30.07.2020 15:01 Sebastian Steininger
function StringConverter:StringToMoodJsonArray(strArray)
    local arrArray = {}
    local nArrayIndex = 1
    strArray = string.sub(strArray, 2,-2)
    for i=1,string.len(strArray),1 do
	   if(string.sub(strArray, i,i) == "{") then
		  for j=i+1, string.len(strArray),1 do
			 if(string.sub(strArray, j,j)=="}") then
				arrArray[nArrayIndex] = string.sub(strArray,i,j)
				Dbg:Info("arrArray[nArrayIndex]: "..arrArray[nArrayIndex])
				nArrayIndex = nArrayIndex+1
				i=j
				break
			 end
		  end
	   end
    end
    Dbg:Trace("return array")
    return arrArray, nArrayIndex-1
end