require ('converter')

LightControllerV2 = LightControllerV2 or {}

--@LightControllerV2:new creates a new LightControllerV2 object
--@param communicator is the communicator object
--@param loxApp3JsonHandler is the jsonHandler object
--@return a instance of it selfe
--created 30.07.2020 11:05 Sebastian Steininger, changed 12.08.2020 12:45 Sebastian Steininger
function LightControllerV2:new(communicator)
    local objLightControllerV2 = {}
    setmetatable(objLightControllerV2, self)
    self.__index = self
    objLightControllerV2.communicator = communicator
    objLightControllerV2.uuid = Properties["uuid"]
    objLightControllerV2.moodId = nil
    --LightControllerV2:RegisterReceiveFunctions()
    return objLightControllerV2
end

--@LightControllerV2:RegisterReceiveFunctions registers the receive functions
--created 12.08.2020 12:45 Sebastian Steininger, changed 12.08.2020 12:45 Sebastian Steininger
function LightControllerV2:RegisterReceiveFunctions()
    local fncActiveMoodChange = function (message)
	   self:ActiveMoodChanged(message)
    end
    communicator:RegisterReceiveFunction(Properties["activeMoods"], fncActiveMoodChange)
    local fncGetLightningScenes = function (message)
	   self:GetLightningScenes(message)
    end
    communicator:RegisterReceiveFunction(Properties["moodList"], fncGetLightningScenes)
end

--@LightControllerV2:ChangeMood changes the lightning mood
--@param moodid is the id of the mood
--created 30.07.2020 11:05 Sebastian Steininger, changed 03.08.2020 10:40 Sebastian Steininger
function LightControllerV2:ChangeMood(moodId)
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/changeTo/"..tostring(moodId))
end

--@LightControllerV2:MoodPlus changes to the next Mood
--created 30.07.2020 11:05 Sebastian Steininger, changed 03.08.2020 10:40 Sebastian Steininger
function LightControllerV2:MoodPlus()
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/plus")
end

--@LightControllerV2:MoodMinus changes to the last Mood
--created 30.07.2020 11:05 Sebastian Steininger, changed 03.08.2020 10:40 Sebastian Steininger
function LightControllerV2:MoodMinus()
    self.communicator:SendCommand("jdev/sps/io/"..Properties["uuid"].."/minus")
end

--@ActiveMoodChanged is called on the registered event on mood change
--created 30.07.2020 13:00 Sebastian Steininger, changed 31.07.2020 09:24 Sebastian Steininger
function LightControllerV2:ActiveMoodChanged(message)
    self.moodId = string.sub(message.strText, 2,-2)
    if(self.nNumMoods) then
	   Dbg:Info("moodChanged (LightControllerv2")
	   Dbg:Info(message.strText)
	   local arrMoodIDs, nNumActiveMoods = StringConverter:StringToNumArray(message.strText)
	   local strMoodNames =""
	   for h=1,nNumActiveMoods,1 do
		  Dbg:Info("Try to match "..tostring(h).." id to a name")
		  for i=1,self.nNumMoods,1 do
			 Dbg:Trace("Try to Match: "..arrMoodIDs[h])
			 if (tonumber(self.moodArray[i].id)  == tonumber(arrMoodIDs[h])) then
				strMoodNames = strMoodNames..self.moodArray[i].name..","
				break
			 end
		  end
		  self:FireEventByID(arrMoodIDs[h])
	   end
	   self.arrLastActiveMoods = arrMoodIDs
	   self.nLastNumMoodsLen = nNumActiveMoods
	   local strMoodNames = string.sub(strMoodNames,1,-2)
	   Dbg:Info("moodNames: "..strMoodNames)
	   C4:UpdateProperty("LightningScene", strMoodNames)
    end
end

--@GetLightningScenes is called on the registered event on get lightningScene
--created 30.07.2020 14:38 Sebastian Steininger, changed 07.08.2020 09:56 Sebastian Steininger
function LightControllerV2:GetLightningScenes(message)
    Dbg:Trace("got moodlist")
    Dbg:Debug(message.strText)
    local arrMoodJson = {}
    Dbg:Debug("testStringToMoodJsonArray")
    arrMoodJson, self.nNumMoods = StringConverter:StringToMoodJsonArray(message.strText)
    self.moodArray = {}
    self.arrMoodIdMap ={}
    Dbg:Trace("json to table")
    for i=1,self.nNumMoods,1 do
	   Dbg:Debug("try json Decode")
	   self.moodArray[i] = C4:JsonDecode(arrMoodJson[i])
	   self.arrMoodIdMap[tonumber(self.moodArray[i].id)] = i
    end
    for i=1,self.nNumMoods,1 do
	   print("EventID "..tostring(i)..": "..tostring(self.moodArray[i].id)..", "..tostring(self.moodArray[i].name))
    end
    if(self.moodId~=nil)then
	   local message = {}
	   message.strText = "["..self.moodId.."]"
	   self:ActiveMoodChanged(message)
    end
end

--@ActiveMoodChanged is called on the registered event on get lightningScene
--created 30.07.2020 17:10 Sebastian Steininger, changed 31.07.2020 09:35 Sebastian Steininger
function LightControllerV2:FireEventByID(nMoodID)
    Dbg:Debug(nMoodID)
    Dbg:Trace("nMoodID: "..tostring(nMoodID))
    
    if((not(self.arrLastActiveMoods)) or (TableMoodsContains(self.arrLastActiveMoods,self.nLastNumMoodsLen, nMoodID)==false)) then
	   Dbg:Trace("fireEvent")
	   C4:FireEventByID(self.arrMoodIdMap[tonumber(nMoodID)])
	   Dbg:Debug("event fired")
    end
end

--@TableContains check if a table contains a value
--created 31.07.2020 09:00 Sebastian Steininger, changed 07.08.2020 10:37 Sebastian Steininger
function TableMoodsContains(tab, len, value)
    Dbg:Debug("checkIFTable contains")
    for i=1,len,1 do
	   if(tostring(tab[i]) == tostring(value)) then
		  Dbg:Debug("TableMoodsContains = true")
		  return true
	   end
    end
    Dbg:Debug("TableMoodsContains = false")
    return false
end

function LightControllerV2:ShowEventMap()
    if type(self.nNumMoods) == "number" then
        for i=1,self.nNumMoods,1 do
            print("EventID "..tostring(i)..": "..tostring(self.moodArray[i].id)..", "..tostring(self.moodArray[i].name))
        end
    else
        Dbg:Trace("No moodes found.")
    end
end